# Repository path:    $HeadURL: https://90.145.156.222/DMS-Houtribdijk/code/python-ftp-svn/source/MonitorStatus.py $
# Last committed:     $Revision: 279039 $; $Author: jansink $; $Date: 2019-08-19 11:31:01 +0200 (ma, 19 aug 2019) $
#
from misc.Utils import Utils
from dmsLogger.DmsLogger import DmsLogger
from fewsServices.ReadFewsPy import ReadFewsPy

import pandas as pd
import json
import sys
import getopt


def locations_sensors(settings):
    logger = DmsLogger.set_logger(
        settings["log_folder"], settings["logger"], settings["logfile"]
    )
    logger.info("Run_MonitorStatus")

    # pi url
    pi_url = settings["pi_url"]
    filter_id = settings["filter_id"]
    par_ids = settings["parameter_ids"]
    loc_ids = settings["location_ids"]
    status_overview = settings["status_overview"]
    output_location = settings["output_location"]

    client_timezone = "Etc/GMT"

    # create rounded reference T0
    T0 = pd.Timestamp.now().round("60min").tz_localize("Etc/GMT")
    # T0 = T0.tz_convert("Etc/GMT")  # to timezone as in FEWS

    # and start- / end-date.
    start_date = T0 - pd.Timedelta(hours=3)
    end_date = T0 - pd.Timedelta(hours=2, minutes=10)

    reader = ReadFewsPy(logger, pi_url, filter_id)
    df = reader.read_timeseries(
        logger, filter_id, par_ids, loc_ids, start_date, end_date, client_timezone
    )
    # mask nodata values using the flag value
    df.value.mask(df.flag == 8, inplace=True)

    # apply join on parameter names
    lookup_pars = reader.read_parameters(logger=logger, filter_id=filter_id)
    lookup_pars = lookup_pars.T[["id", "name"]]
    df_merge = df.reset_index().merge(lookup_pars, left_on="parameterId", right_on="id")

    # get overview at which location parameter combination status information is expected
    df_overzicht = pd.read_csv(status_overview, index_col=0)

    # parse to combination data
    combinations = []
    for row in df_overzicht.iterrows():
        for idx, item in enumerate(row[1]):
            pair = {"locId": row[0], "parId": row[1].index[idx], "monitor": int(item)>0}
            combinations.append(pair)

    df_monitor = pd.DataFrame.from_dict(combinations)

    # apply inner-join on retrieved status information and overview of status setting
    df_status = df_merge.merge(
        df_monitor, left_on=["locationId", "name"], right_on=["locId", "parId"]
    )
    df_status = df_status[["date", "locationId", "name", "monitor", "value"]]

    # only select location parameter combinations where there is monitoring activated
    df_msg = df_status[df_status["monitor"] == 1]

    # filter where status is NaN or 0 (only value -1 is valid)
    msg_filter = [any(tup) for tup in zip(df_msg.value.isnull(), df_msg.value == 0)]

    # add columns for prtg
    df_msg.loc[:, "channel"] = df_msg["locationId"] + " - " + df_msg["name"]
    df_msg.loc[:, "value_error"] = 0
    df_msg.loc[msg_filter, "value_error"] = 2
    df_msg.loc[:, "limitmaxerror"] = 1
    df_msg.loc[:, "limitmode"] = 1
    df_msg.loc[
        :, "limiterrormsg"
    ] = "Status-tijdreeks locatie-sensor combinatie wordt correct ontvangen (waarde -1)"
    df_msg.loc[
        msg_filter, "limiterrormsg"
    ] = "Status-tijdreeks wordt niet ontvangen (waarde 0 of NaN) terwijl de locatie-sensor combinatie wel is aangemeld. ({})".format(
        df_msg.date.dt.strftime("%Y-%m-%d %H:%M")
    )

    # filter columns to output
    df_msg_out = df_msg[
        ["channel", "value_error", "limitmaxerror", "limitmode", "limiterrormsg"]
    ]
    df_msg_out.columns = [
        "channel",
        "value",
        "limitmaxerror",
        "limitmode",
        "limiterrormsg",
    ]

    # save to file
    monitor_status_out = {"prtg": {"result": df_msg_out.to_dict(orient="records")}}
    with open(output_location, "w") as f:
        print(json.dumps(monitor_status_out), file=f)


def main(argv):
    # input argument checking
    try:
        opts, args = getopt.getopt(argv, "hs:", ["settings="])
    except getopt.GetoptError:
        print("MonitorStatus.py -s <settings_file>")
        sys.exit(2)
    for opt, arg in opts:
        if opt == "-h":
            print("MonitorStatus.py -s <settings_file>")
            sys.exit()
        elif opt in ("-s", "--settings"):
            settings_file = arg

    try:
        # settings_file = "../connections/monitor_status_locs_sensors.json"
        settings = Utils.read_monitor_status_locs_sensors_settings(settings_file)
        locations_sensors(settings)
    except FileNotFoundError:
        exit(1)


if __name__ == "__main__":
    main(sys.argv[1:])
    # fire.Fire()
